

class ConstellationNumberError(Exception):
    pass


class ConstellationConfigurationError(Exception):
    pass


class ConstellationPlaneMismatchError(Exception):
    pass


class PhaseError(Exception):
    pass


class InclinationError(Exception):
    pass


class AltitudeError(Exception):
    pass


class EccentricityError(Exception):
    pass


class BeamError(Exception):
    pass


class FocusError(Exception):
    pass
